/*
 *
 *  This file contains the entry point for the application.
 *  The name of this entry point is compiler dependent.
 *  It jumps to the BSP which is responsible for performing
 *  all initialization.
 *
 *  COPYRIGHT (c) 1989-1998.
 *  On-Line Applications Research Corporation (OAR).
 *  Copyright assigned to U.S. Government, 1994.
 *
 *  The license and distribution terms for this file may in
 *  the file LICENSE in this distribution or at
 *  http://www.OARcorp.com/rtems/license.html.
 *
 * Based on the `gen68302' board support package, and covered by the
 * original distribution terms.
 *
 * W. Eric Norum
 * Saskatchewan Accelerator Laboratory
 * University of Saskatchewan
 * Saskatoon, Saskatchewan, CANADA
 * eric@skatter.usask.ca
 *
 *  $Id: start360.s,v 1.11 1998/02/17 23:44:51 joel Exp $
 */

#include "asm.h"

BEGIN_CODE


/*
 *  For some reason, the symbol start must not be global.
 *
 *       .global start
 */

/*
 * Initial PC
 */
.globl start
start:
	/*
	 * Step 1: Setup the stack pointer - replace this code with proper HW init code
	 *         which uses onboard SRAM 
	 */
	movel	#stack_init,a7		| set stack pointer (stack_init is set in linker script)
	
	/*
	 * Remaining steps are handled by C code
	 */
	jmp	SYM(_Init5307)		| Start C code (which never returns)
	
/*
 * Copy DATA segment, clear BSS segment, initialize heap, start C program.
 * Assume that DATA and BSS sizes are multiples of 4.
 */
	PUBLIC (_CopyDataClearBSSAndStart)
SYM(_CopyDataClearBSSAndStart):
	lea	copy_start,a0		| Get start of DATA in RAM
	lea	SYM(etext),a2		| Get start of DATA in ROM
	cmpl	a0,a2			| Are they the same?
	beq.s	NOCOPY			| Yes, no copy necessary
	lea	copy_end,a1		| Get end of DATA in RAM
	bra.s	COPYLOOPTEST		| Branch into copy loop
COPYLOOP:
	movel	a2@+,a0@+		| Copy word from ROM to RAM
COPYLOOPTEST:
	cmpl	a1,a0			| Done?
	bcs.s	COPYLOOP		| No, skip
NOCOPY:

	lea	clear_start,a0		| Get start of BSS
	lea	clear_end,a1		| Get end of BSS
	clrl	d0			| Value to set
	bra.s	ZEROLOOPTEST		| Branch into clear loop
ZEROLOOP:
	movel	d0,a0@+			| Clear a word
ZEROLOOPTEST:
	cmpl	a1,a0			| Done?
	bcs.s	ZEROLOOP		| No, skip

	jsr	SYM(boot_card)		| Call C main
 

	/*
	 * Right : Now we're ready to boot RTEMS
	 */
	jsr	SYM(boot_card)		| Call C boot_card function to startup RTEMS

        .align 2
	PUBLIC (_HeapSize)
SYM (_HeapSize):
        .long  HeapSize
	PUBLIC (_StackSize)
SYM (_StackSize):
        .long  StackSize
	PUBLIC (sim)
SYM (sim):
        .long  MBase |Internal module base address typecast with SIM5307 to access
END_CODE

END

